part of 'blocs.dart';

abstract class BaseEvent {
  const BaseEvent({String? tag}) : this._tag = tag;

  /// Defined ID for this event, will be used for handle event & state
  final String? _tag;

  String get tag => _tag ?? runtimeType.toString();

  /// Override this getter if this event need connect to remote data source
  bool get needCheckConnection => false;
}

class BaseLoadingEvent extends BaseEvent {
  const BaseLoadingEvent({super.tag});
}

class ResetBlocEvent extends BaseEvent {
  const ResetBlocEvent({
    this.needToCancelRunningTask = true,
    this.needToNotify = false,
  });

  final bool needToCancelRunningTask;
  final bool needToNotify;
}

class CancelRunningTaskEvent extends BaseEvent {}

class BasePagingEvent<Q> extends BaseEvent {
  const BasePagingEvent({
    this.queryData,
    this.limit = defaultLimitPaging,
    this.isFirstRequest = false,
    this.needLoading = true,
    super.tag,
  });

  final Q? queryData;
  final int limit;
  final bool isFirstRequest;
  final bool needLoading;
}

class BaseErrorEvent extends BaseEvent {
  const BaseErrorEvent({
    required this.exception,
    super.tag,
  });

  final KntException exception;
}
