part of 'blocs.dart';

abstract class BaseState {
  const BaseState({String? tag}) : this._tag = tag;

  /// Defined ID for this state, will be used for handle event & state
  final String? _tag;

  String get tag => _tag ?? runtimeType.toString();
}

class BaseInitState extends BaseState {
  const BaseInitState({super.tag});
}

class BaseResetDoneState extends BaseState {
  const BaseResetDoneState({super.tag});
}

class BaseLoadingState extends BaseState {
  const BaseLoadingState({super.tag});
}

class BaseEmptyResultState extends BaseState {
  const BaseEmptyResultState({super.tag});
}

class BaseHasResultState<D> extends BaseState {
  const BaseHasResultState(
    this.data, {
    super.tag,
  });

  final D data;

  @override
  String toString() => '$runtimeType { tag: $tag, data: ${data?.toString()} }';
}

class BasePagingResultState<D> extends BaseHasResultState<List<D>> {
  const BasePagingResultState(
    List<D> dataList,
    this.currentOffset, {
    super.tag,
  }) : super(dataList);

  final int currentOffset;
}

class BaseErrorState extends BaseState {
  const BaseErrorState({
    required this.exception,
    super.tag,
  });

  final KntException exception;

  @override
  String toString() => '$runtimeType { tag: $tag, exception: $exception }';
}
