import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/services.dart' show rootBundle;
import 'package:knt_bloc/knt_bloc.dart';

import '../model/book.dart';

class BookRepo {
  Future<List<Book>> fetchBooks({
    bool simulateLongData = true,
    bool simulateException = false,
  }) async {
    final books = <Book>[];
    final loadTimes = simulateLongData ? _loadTimesWillMakeLagging : 1;
    for (var index = 0; index < loadTimes; index++) {
      final loadedBook = await loadBooks(BookSource.string);
      if (index == loadTimes - 1) {
        books.addAll(loadedBook);
      }
      if (simulateException && index == 1000) {
        throw const KntException(
          KntIssue.undefined,
          additionalInfo: 'Corrupted books!',
        );
      }
    }
    return books;
  }

  Future<List<Book>> loadBooks(BookSource source) async {
    switch (source) {
      case BookSource.string:
        return List<Book>.from(
            jsonDecode(_booksJson).map((model) => Book.fromJson(model)));
      case BookSource.github:
        return await Dio()
            .get(
                'https://raw.githubusercontent.com/benoitvallon/100-best-books/master/books.json')
            .then(
              (response) => List<Book>.from(
                jsonDecode(response.data).map((model) => Book.fromJson(model)),
              ),
            );
      case BookSource.assetsBundle:
        return await rootBundle.loadString('assets/books.json').then(
              (jsonStr) => List<Book>.from(
                jsonDecode(jsonStr).map((model) => Book.fromJson(model)),
              ),
            );
    }
  }
}

enum BookSource {
  string,
  github,
  assetsBundle,
}

const _loadTimesWillMakeLagging = 100000;
const _booksJson =
    "[{\"author\":\"ChinuaAchebe\",\"country\":\"Nigeria\",\"imageLink\":\"images/things-fall-apart.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Things_Fall_Apart\\n\",\"pages\":209,\"title\":\"ThingsFallApart\",\"year\":1958},{\"author\":\"HansChristianAndersen\",\"country\":\"Denmark\",\"imageLink\":\"images/fairy-tales.jpg\",\"language\":\"Danish\",\"link\":\"https://en.wikipedia.org/wiki/Fairy_Tales_Told_for_Children._First_Collection.\\n\",\"pages\":784,\"title\":\"Fairytales\",\"year\":1836},{\"author\":\"DanteAlighieri\",\"country\":\"Italy\",\"imageLink\":\"images/the-divine-comedy.jpg\",\"language\":\"Italian\",\"link\":\"https://en.wikipedia.org/wiki/Divine_Comedy\\n\",\"pages\":928,\"title\":\"TheDivineComedy\",\"year\":1315},{\"author\":\"Unknown\",\"country\":\"SumerandAkkadianEmpire\",\"imageLink\":\"images/the-epic-of-gilgamesh.jpg\",\"language\":\"Akkadian\",\"link\":\"https://en.wikipedia.org/wiki/Epic_of_Gilgamesh\\n\",\"pages\":160,\"title\":\"TheEpicOfGilgamesh\",\"year\":-1700},{\"author\":\"Unknown\",\"country\":\"AchaemenidEmpire\",\"imageLink\":\"images/the-book-of-job.jpg\",\"language\":\"Hebrew\",\"link\":\"https://en.wikipedia.org/wiki/Book_of_Job\\n\",\"pages\":176,\"title\":\"TheBookOfJob\",\"year\":-600},{\"author\":\"Unknown\",\"country\":\"India/Iran/Iraq/Egypt/Tajikistan\",\"imageLink\":\"images/one-thousand-and-one-nights.jpg\",\"language\":\"Arabic\",\"link\":\"https://en.wikipedia.org/wiki/One_Thousand_and_One_Nights\\n\",\"pages\":288,\"title\":\"OneThousandandOneNights\",\"year\":1200},{\"author\":\"Unknown\",\"country\":\"Iceland\",\"imageLink\":\"images/njals-saga.jpg\",\"language\":\"OldNorse\",\"link\":\"https://en.wikipedia.org/wiki/Nj%C3%A1ls_saga\\n\",\"pages\":384,\"title\":\"Nj\\u00e1l'sSaga\",\"year\":1350},{\"author\":\"JaneAusten\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/pride-and-prejudice.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Pride_and_Prejudice\\n\",\"pages\":226,\"title\":\"PrideandPrejudice\",\"year\":1813},{\"author\":\"Honor\\u00e9deBalzac\",\"country\":\"France\",\"imageLink\":\"images/le-pere-goriot.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Le_P%C3%A8re_Goriot\\n\",\"pages\":443,\"title\":\"LeP\\u00e8reGoriot\",\"year\":1835},{\"author\":\"SamuelBeckett\",\"country\":\"RepublicofIreland\",\"imageLink\":\"images/molloy-malone-dies-the-unnamable.jpg\",\"language\":\"French,English\",\"link\":\"https://en.wikipedia.org/wiki/Molloy_(novel)\\n\",\"pages\":256,\"title\":\"Molloy,MaloneDies,TheUnnamable,thetrilogy\",\"year\":1952},{\"author\":\"GiovanniBoccaccio\",\"country\":\"Italy\",\"imageLink\":\"images/the-decameron.jpg\",\"language\":\"Italian\",\"link\":\"https://en.wikipedia.org/wiki/The_Decameron\\n\",\"pages\":1024,\"title\":\"TheDecameron\",\"year\":1351},{\"author\":\"JorgeLuisBorges\",\"country\":\"Argentina\",\"imageLink\":\"images/ficciones.jpg\",\"language\":\"Spanish\",\"link\":\"https://en.wikipedia.org/wiki/Ficciones\\n\",\"pages\":224,\"title\":\"Ficciones\",\"year\":1965},{\"author\":\"EmilyBront\\u00eb\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/wuthering-heights.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Wuthering_Heights\\n\",\"pages\":342,\"title\":\"WutheringHeights\",\"year\":1847},{\"author\":\"AlbertCamus\",\"country\":\"Algeria,FrenchEmpire\",\"imageLink\":\"images/l-etranger.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/The_Stranger_(novel)\\n\",\"pages\":185,\"title\":\"TheStranger\",\"year\":1942},{\"author\":\"PaulCelan\",\"country\":\"Romania,France\",\"imageLink\":\"images/poems-paul-celan.jpg\",\"language\":\"German\",\"link\":\"\\n\",\"pages\":320,\"title\":\"Poems\",\"year\":1952},{\"author\":\"Louis-FerdinandC\\u00e9line\",\"country\":\"France\",\"imageLink\":\"images/voyage-au-bout-de-la-nuit.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Journey_to_the_End_of_the_Night\\n\",\"pages\":505,\"title\":\"JourneytotheEndoftheNight\",\"year\":1932},{\"author\":\"MigueldeCervantes\",\"country\":\"Spain\",\"imageLink\":\"images/don-quijote-de-la-mancha.jpg\",\"language\":\"Spanish\",\"link\":\"https://en.wikipedia.org/wiki/Don_Quixote\\n\",\"pages\":1056,\"title\":\"DonQuijoteDeLaMancha\",\"year\":1610},{\"author\":\"GeoffreyChaucer\",\"country\":\"England\",\"imageLink\":\"images/the-canterbury-tales.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/The_Canterbury_Tales\\n\",\"pages\":544,\"title\":\"TheCanterburyTales\",\"year\":1450},{\"author\":\"AntonChekhov\",\"country\":\"Russia\",\"imageLink\":\"images/stories-of-anton-chekhov.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/List_of_short_stories_by_Anton_Chekhov\\n\",\"pages\":194,\"title\":\"Stories\",\"year\":1886},{\"author\":\"JosephConrad\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/nostromo.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Nostromo\\n\",\"pages\":320,\"title\":\"Nostromo\",\"year\":1904},{\"author\":\"CharlesDickens\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/great-expectations.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Great_Expectations\\n\",\"pages\":194,\"title\":\"GreatExpectations\",\"year\":1861},{\"author\":\"DenisDiderot\",\"country\":\"France\",\"imageLink\":\"images/jacques-the-fatalist.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Jacques_the_Fatalist\\n\",\"pages\":596,\"title\":\"JacquestheFatalist\",\"year\":1796},{\"author\":\"AlfredD\\u00f6blin\",\"country\":\"Germany\",\"imageLink\":\"images/berlin-alexanderplatz.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/Berlin_Alexanderplatz\\n\",\"pages\":600,\"title\":\"BerlinAlexanderplatz\",\"year\":1929},{\"author\":\"FyodorDostoevsky\",\"country\":\"Russia\",\"imageLink\":\"images/crime-and-punishment.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/Crime_and_Punishment\\n\",\"pages\":551,\"title\":\"CrimeandPunishment\",\"year\":1866},{\"author\":\"FyodorDostoevsky\",\"country\":\"Russia\",\"imageLink\":\"images/the-idiot.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/The_Idiot\\n\",\"pages\":656,\"title\":\"TheIdiot\",\"year\":1869},{\"author\":\"FyodorDostoevsky\",\"country\":\"Russia\",\"imageLink\":\"images/the-possessed.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/Demons_(Dostoyevsky_novel)\\n\",\"pages\":768,\"title\":\"ThePossessed\",\"year\":1872},{\"author\":\"FyodorDostoevsky\",\"country\":\"Russia\",\"imageLink\":\"images/the-brothers-karamazov.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/The_Brothers_Karamazov\\n\",\"pages\":824,\"title\":\"TheBrothersKaramazov\",\"year\":1880},{\"author\":\"GeorgeEliot\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/middlemarch.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Middlemarch\\n\",\"pages\":800,\"title\":\"Middlemarch\",\"year\":1871},{\"author\":\"RalphEllison\",\"country\":\"UnitedStates\",\"imageLink\":\"images/invisible-man.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Invisible_Man\\n\",\"pages\":581,\"title\":\"InvisibleMan\",\"year\":1952},{\"author\":\"Euripides\",\"country\":\"Greece\",\"imageLink\":\"images/medea.jpg\",\"language\":\"Greek\",\"link\":\"https://en.wikipedia.org/wiki/Medea_(play)\\n\",\"pages\":104,\"title\":\"Medea\",\"year\":-431},{\"author\":\"WilliamFaulkner\",\"country\":\"UnitedStates\",\"imageLink\":\"images/absalom-absalom.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Absalom,_Absalom!\\n\",\"pages\":313,\"title\":\"Absalom,Absalom!\",\"year\":1936},{\"author\":\"WilliamFaulkner\",\"country\":\"UnitedStates\",\"imageLink\":\"images/the-sound-and-the-fury.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/The_Sound_and_the_Fury\\n\",\"pages\":326,\"title\":\"TheSoundandtheFury\",\"year\":1929},{\"author\":\"GustaveFlaubert\",\"country\":\"France\",\"imageLink\":\"images/madame-bovary.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Madame_Bovary\\n\",\"pages\":528,\"title\":\"MadameBovary\",\"year\":1857},{\"author\":\"GustaveFlaubert\",\"country\":\"France\",\"imageLink\":\"images/l-education-sentimentale.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Sentimental_Education\\n\",\"pages\":606,\"title\":\"SentimentalEducation\",\"year\":1869},{\"author\":\"FedericoGarc\\u00edaLorca\",\"country\":\"Spain\",\"imageLink\":\"images/gypsy-ballads.jpg\",\"language\":\"Spanish\",\"link\":\"https://en.wikipedia.org/wiki/Gypsy_Ballads\\n\",\"pages\":218,\"title\":\"GypsyBallads\",\"year\":1928},{\"author\":\"GabrielGarc\\u00edaM\\u00e1rquez\",\"country\":\"Colombia\",\"imageLink\":\"images/one-hundred-years-of-solitude.jpg\",\"language\":\"Spanish\",\"link\":\"https://en.wikipedia.org/wiki/One_Hundred_Years_of_Solitude\\n\",\"pages\":417,\"title\":\"OneHundredYearsofSolitude\",\"year\":1967},{\"author\":\"GabrielGarc\\u00edaM\\u00e1rquez\",\"country\":\"Colombia\",\"imageLink\":\"images/love-in-the-time-of-cholera.jpg\",\"language\":\"Spanish\",\"link\":\"https://en.wikipedia.org/wiki/Love_in_the_Time_of_Cholera\\n\",\"pages\":368,\"title\":\"LoveintheTimeofCholera\",\"year\":1985},{\"author\":\"JohannWolfgangvonGoethe\",\"country\":\"Saxe-Weimar\",\"imageLink\":\"images/faust.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/Goethe%27s_Faust\\n\",\"pages\":158,\"title\":\"Faust\",\"year\":1832},{\"author\":\"NikolaiGogol\",\"country\":\"Russia\",\"imageLink\":\"images/dead-souls.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/Dead_Souls\\n\",\"pages\":432,\"title\":\"DeadSouls\",\"year\":1842},{\"author\":\"G\\u00fcnterGrass\",\"country\":\"Germany\",\"imageLink\":\"images/the-tin-drum.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/The_Tin_Drum\\n\",\"pages\":600,\"title\":\"TheTinDrum\",\"year\":1959},{\"author\":\"Jo\\u00e3oGuimar\\u00e3esRosa\",\"country\":\"Brazil\",\"imageLink\":\"images/the-devil-to-pay-in-the-backlands.jpg\",\"language\":\"Portuguese\",\"link\":\"https://en.wikipedia.org/wiki/The_Devil_to_Pay_in_the_Backlands\\n\",\"pages\":494,\"title\":\"TheDeviltoPayintheBacklands\",\"year\":1956},{\"author\":\"KnutHamsun\",\"country\":\"Norway\",\"imageLink\":\"images/hunger.jpg\",\"language\":\"Norwegian\",\"link\":\"https://en.wikipedia.org/wiki/Hunger_(Hamsun_novel)\\n\",\"pages\":176,\"title\":\"Hunger\",\"year\":1890},{\"author\":\"ErnestHemingway\",\"country\":\"UnitedStates\",\"imageLink\":\"images/the-old-man-and-the-sea.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/The_Old_Man_and_the_Sea\\n\",\"pages\":128,\"title\":\"TheOldManandtheSea\",\"year\":1952},{\"author\":\"Homer\",\"country\":\"Greece\",\"imageLink\":\"images/the-iliad-of-homer.jpg\",\"language\":\"Greek\",\"link\":\"https://en.wikipedia.org/wiki/Iliad\\n\",\"pages\":608,\"title\":\"Iliad\",\"year\":-735},{\"author\":\"Homer\",\"country\":\"Greece\",\"imageLink\":\"images/the-odyssey-of-homer.jpg\",\"language\":\"Greek\",\"link\":\"https://en.wikipedia.org/wiki/Odyssey\\n\",\"pages\":374,\"title\":\"Odyssey\",\"year\":-800},{\"author\":\"HenrikIbsen\",\"country\":\"Norway\",\"imageLink\":\"images/a-Dolls-house.jpg\",\"language\":\"Norwegian\",\"link\":\"https://en.wikipedia.org/wiki/A_Doll%27s_House\\n\",\"pages\":68,\"title\":\"ADoll'sHouse\",\"year\":1879},{\"author\":\"JamesJoyce\",\"country\":\"IrishFreeState\",\"imageLink\":\"images/ulysses.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Ulysses_(novel)\\n\",\"pages\":228,\"title\":\"Ulysses\",\"year\":1922},{\"author\":\"FranzKafka\",\"country\":\"Czechoslovakia\",\"imageLink\":\"images/stories-of-franz-kafka.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/Franz_Kafka_bibliography#Short_stories\\n\",\"pages\":488,\"title\":\"Stories\",\"year\":1924},{\"author\":\"FranzKafka\",\"country\":\"Czechoslovakia\",\"imageLink\":\"images/the-trial.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/The_Trial\\n\",\"pages\":160,\"title\":\"TheTrial\",\"year\":1925},{\"author\":\"FranzKafka\",\"country\":\"Czechoslovakia\",\"imageLink\":\"images/the-castle.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/The_Castle_(novel)\\n\",\"pages\":352,\"title\":\"TheCastle\",\"year\":1926},{\"author\":\"K\\u0101lid\\u0101sa\",\"country\":\"India\",\"imageLink\":\"images/the-recognition-of-shakuntala.jpg\",\"language\":\"Sanskrit\",\"link\":\"https://en.wikipedia.org/wiki/Abhij%C3%B1%C4%81na%C5%9B%C4%81kuntalam\\n\",\"pages\":147,\"title\":\"TherecognitionofShakuntala\",\"year\":150},{\"author\":\"YasunariKawabata\",\"country\":\"Japan\",\"imageLink\":\"images/the-sound-of-the-mountain.jpg\",\"language\":\"Japanese\",\"link\":\"https://en.wikipedia.org/wiki/The_Sound_of_the_Mountain\\n\",\"pages\":288,\"title\":\"TheSoundoftheMountain\",\"year\":1954},{\"author\":\"NikosKazantzakis\",\"country\":\"Greece\",\"imageLink\":\"images/zorba-the-greek.jpg\",\"language\":\"Greek\",\"link\":\"https://en.wikipedia.org/wiki/Zorba_the_Greek\\n\",\"pages\":368,\"title\":\"ZorbatheGreek\",\"year\":1946},{\"author\":\"D.H.Lawrence\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/sons-and-lovers.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Sons_and_Lovers\\n\",\"pages\":432,\"title\":\"SonsandLovers\",\"year\":1913},{\"author\":\"Halld\\u00f3rLaxness\",\"country\":\"Iceland\",\"imageLink\":\"images/independent-people.jpg\",\"language\":\"Icelandic\",\"link\":\"https://en.wikipedia.org/wiki/Independent_People\\n\",\"pages\":470,\"title\":\"IndependentPeople\",\"year\":1934},{\"author\":\"GiacomoLeopardi\",\"country\":\"Italy\",\"imageLink\":\"images/poems-giacomo-leopardi.jpg\",\"language\":\"Italian\",\"link\":\"\\n\",\"pages\":184,\"title\":\"Poems\",\"year\":1818},{\"author\":\"DorisLessing\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/the-golden-notebook.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/The_Golden_Notebook\\n\",\"pages\":688,\"title\":\"TheGoldenNotebook\",\"year\":1962},{\"author\":\"AstridLindgren\",\"country\":\"Sweden\",\"imageLink\":\"images/pippi-longstocking.jpg\",\"language\":\"Swedish\",\"link\":\"https://en.wikipedia.org/wiki/Pippi_Longstocking\\n\",\"pages\":160,\"title\":\"PippiLongstocking\",\"year\":1945},{\"author\":\"LuXun\",\"country\":\"China\",\"imageLink\":\"images/diary-of-a-madman.jpg\",\"language\":\"Chinese\",\"link\":\"https://en.wikipedia.org/wiki/A_Madman%27s_Diary\\n\",\"pages\":389,\"title\":\"DiaryofaMadman\",\"year\":1918},{\"author\":\"NaguibMahfouz\",\"country\":\"Egypt\",\"imageLink\":\"images/children-of-gebelawi.jpg\",\"language\":\"Arabic\",\"link\":\"https://en.wikipedia.org/wiki/Children_of_Gebelawi\\n\",\"pages\":355,\"title\":\"ChildrenofGebelawi\",\"year\":1959},{\"author\":\"ThomasMann\",\"country\":\"Germany\",\"imageLink\":\"images/buddenbrooks.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/Buddenbrooks\\n\",\"pages\":736,\"title\":\"Buddenbrooks\",\"year\":1901},{\"author\":\"ThomasMann\",\"country\":\"Germany\",\"imageLink\":\"images/the-magic-mountain.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/The_Magic_Mountain\\n\",\"pages\":720,\"title\":\"TheMagicMountain\",\"year\":1924},{\"author\":\"HermanMelville\",\"country\":\"UnitedStates\",\"imageLink\":\"images/moby-dick.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Moby-Dick\\n\",\"pages\":378,\"title\":\"MobyDick\",\"year\":1851},{\"author\":\"MicheldeMontaigne\",\"country\":\"France\",\"imageLink\":\"images/essais.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Essays_(Montaigne)\\n\",\"pages\":404,\"title\":\"Essays\",\"year\":1595},{\"author\":\"ElsaMorante\",\"country\":\"Italy\",\"imageLink\":\"images/history.jpg\",\"language\":\"Italian\",\"link\":\"https://en.wikipedia.org/wiki/History_(novel)\\n\",\"pages\":600,\"title\":\"History\",\"year\":1974},{\"author\":\"ToniMorrison\",\"country\":\"UnitedStates\",\"imageLink\":\"images/beloved.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Beloved_(novel)\\n\",\"pages\":321,\"title\":\"Beloved\",\"year\":1987},{\"author\":\"MurasakiShikibu\",\"country\":\"Japan\",\"imageLink\":\"images/the-tale-of-genji.jpg\",\"language\":\"Japanese\",\"link\":\"https://en.wikipedia.org/wiki/The_Tale_of_Genji\\n\",\"pages\":1360,\"title\":\"TheTaleofGenji\",\"year\":1006},{\"author\":\"RobertMusil\",\"country\":\"Austria\",\"imageLink\":\"images/the-man-without-qualities.jpg\",\"language\":\"German\",\"link\":\"https://en.wikipedia.org/wiki/The_Man_Without_Qualities\\n\",\"pages\":365,\"title\":\"TheManWithoutQualities\",\"year\":1931},{\"author\":\"VladimirNabokov\",\"country\":\"Russia/UnitedStates\",\"imageLink\":\"images/lolita.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Lolita\\n\",\"pages\":317,\"title\":\"Lolita\",\"year\":1955},{\"author\":\"GeorgeOrwell\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/nineteen-eighty-four.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Nineteen_Eighty-Four\\n\",\"pages\":272,\"title\":\"NineteenEighty-Four\",\"year\":1949},{\"author\":\"Ovid\",\"country\":\"RomanEmpire\",\"imageLink\":\"images/the-metamorphoses-of-ovid.jpg\",\"language\":\"ClassicalLatin\",\"link\":\"https://en.wikipedia.org/wiki/Metamorphoses\\n\",\"pages\":576,\"title\":\"Metamorphoses\",\"year\":100},{\"author\":\"FernandoPessoa\",\"country\":\"Portugal\",\"imageLink\":\"images/the-book-of-disquiet.jpg\",\"language\":\"Portuguese\",\"link\":\"https://en.wikipedia.org/wiki/The_Book_of_Disquiet\\n\",\"pages\":272,\"title\":\"TheBookofDisquiet\",\"year\":1928},{\"author\":\"EdgarAllanPoe\",\"country\":\"UnitedStates\",\"imageLink\":\"images/tales-and-poems-of-edgar-allan-poe.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Edgar_Allan_Poe_bibliography#Tales\\n\",\"pages\":842,\"title\":\"Tales\",\"year\":1950},{\"author\":\"MarcelProust\",\"country\":\"France\",\"imageLink\":\"images/a-la-recherche-du-temps-perdu.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/In_Search_of_Lost_Time\\n\",\"pages\":2408,\"title\":\"InSearchofLostTime\",\"year\":1920},{\"author\":\"Fran\\u00e7oisRabelais\",\"country\":\"France\",\"imageLink\":\"images/gargantua-and-pantagruel.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Gargantua_and_Pantagruel\\n\",\"pages\":623,\"title\":\"GargantuaandPantagruel\",\"year\":1533},{\"author\":\"JuanRulfo\",\"country\":\"Mexico\",\"imageLink\":\"images/pedro-paramo.jpg\",\"language\":\"Spanish\",\"link\":\"https://en.wikipedia.org/wiki/Pedro_P%C3%A1ramo\\n\",\"pages\":124,\"title\":\"PedroP\\u00e1ramo\",\"year\":1955},{\"author\":\"Rumi\",\"country\":\"SultanateofRum\",\"imageLink\":\"images/the-masnavi.jpg\",\"language\":\"Persian\",\"link\":\"https://en.wikipedia.org/wiki/Masnavi\\n\",\"pages\":438,\"title\":\"TheMasnavi\",\"year\":1236},{\"author\":\"SalmanRushdie\",\"country\":\"UnitedKingdom,India\",\"imageLink\":\"images/midnights-children.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Midnight%27s_Children\\n\",\"pages\":536,\"title\":\"Midnight'sChildren\",\"year\":1981},{\"author\":\"Saadi\",\"country\":\"Persia,PersianEmpire\",\"imageLink\":\"images/bostan.jpg\",\"language\":\"Persian\",\"link\":\"https://en.wikipedia.org/wiki/Bustan_(book)\\n\",\"pages\":298,\"title\":\"Bostan\",\"year\":1257},{\"author\":\"TayebSalih\",\"country\":\"Sudan\",\"imageLink\":\"images/season-of-migration-to-the-north.jpg\",\"language\":\"Arabic\",\"link\":\"https://en.wikipedia.org/wiki/Season_of_Migration_to_the_North\\n\",\"pages\":139,\"title\":\"SeasonofMigrationtotheNorth\",\"year\":1966},{\"author\":\"Jos\\u00e9Saramago\",\"country\":\"Portugal\",\"imageLink\":\"images/blindness.jpg\",\"language\":\"Portuguese\",\"link\":\"https://en.wikipedia.org/wiki/Blindness_(novel)\\n\",\"pages\":352,\"title\":\"Blindness\",\"year\":1995},{\"author\":\"WilliamShakespeare\",\"country\":\"England\",\"imageLink\":\"images/hamlet.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Hamlet\\n\",\"pages\":432,\"title\":\"Hamlet\",\"year\":1603},{\"author\":\"WilliamShakespeare\",\"country\":\"England\",\"imageLink\":\"images/king-lear.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/King_Lear\\n\",\"pages\":384,\"title\":\"KingLear\",\"year\":1608},{\"author\":\"WilliamShakespeare\",\"country\":\"England\",\"imageLink\":\"images/othello.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Othello\\n\",\"pages\":314,\"title\":\"Othello\",\"year\":1609},{\"author\":\"Sophocles\",\"country\":\"Greece\",\"imageLink\":\"images/oedipus-the-king.jpg\",\"language\":\"Greek\",\"link\":\"https://en.wikipedia.org/wiki/Oedipus_the_King\\n\",\"pages\":88,\"title\":\"OedipustheKing\",\"year\":-430},{\"author\":\"Stendhal\",\"country\":\"France\",\"imageLink\":\"images/le-rouge-et-le-noir.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/The_Red_and_the_Black\\n\",\"pages\":576,\"title\":\"TheRedandtheBlack\",\"year\":1830},{\"author\":\"LaurenceSterne\",\"country\":\"England\",\"imageLink\":\"images/the-life-and-opinions-of-tristram-shandy.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/The_Life_and_Opinions_of_Tristram_Shandy,_Gentleman\\n\",\"pages\":640,\"title\":\"TheLifeAndOpinionsofTristramShandy\",\"year\":1760},{\"author\":\"ItaloSvevo\",\"country\":\"Italy\",\"imageLink\":\"images/confessions-of-zeno.jpg\",\"language\":\"Italian\",\"link\":\"https://en.wikipedia.org/wiki/Zeno%27s_Conscience\\n\",\"pages\":412,\"title\":\"ConfessionsofZeno\",\"year\":1923},{\"author\":\"JonathanSwift\",\"country\":\"Ireland\",\"imageLink\":\"images/gullivers-travels.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Gulliver%27s_Travels\\n\",\"pages\":178,\"title\":\"Gulliver'sTravels\",\"year\":1726},{\"author\":\"LeoTolstoy\",\"country\":\"Russia\",\"imageLink\":\"images/war-and-peace.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/War_and_Peace\\n\",\"pages\":1296,\"title\":\"WarandPeace\",\"year\":1867},{\"author\":\"LeoTolstoy\",\"country\":\"Russia\",\"imageLink\":\"images/anna-karenina.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/Anna_Karenina\\n\",\"pages\":864,\"title\":\"AnnaKarenina\",\"year\":1877},{\"author\":\"LeoTolstoy\",\"country\":\"Russia\",\"imageLink\":\"images/the-death-of-ivan-ilyich.jpg\",\"language\":\"Russian\",\"link\":\"https://en.wikipedia.org/wiki/The_Death_of_Ivan_Ilyich\\n\",\"pages\":92,\"title\":\"TheDeathofIvanIlyich\",\"year\":1886},{\"author\":\"MarkTwain\",\"country\":\"UnitedStates\",\"imageLink\":\"images/the-adventures-of-huckleberry-finn.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Adventures_of_Huckleberry_Finn\\n\",\"pages\":224,\"title\":\"TheAdventuresofHuckleberryFinn\",\"year\":1884},{\"author\":\"Valmiki\",\"country\":\"India\",\"imageLink\":\"images/ramayana.jpg\",\"language\":\"Sanskrit\",\"link\":\"https://en.wikipedia.org/wiki/Ramayana\\n\",\"pages\":152,\"title\":\"Ramayana\",\"year\":-450},{\"author\":\"Virgil\",\"country\":\"RomanEmpire\",\"imageLink\":\"images/the-aeneid.jpg\",\"language\":\"ClassicalLatin\",\"link\":\"https://en.wikipedia.org/wiki/Aeneid\\n\",\"pages\":442,\"title\":\"TheAeneid\",\"year\":-23},{\"author\":\"Vyasa\",\"country\":\"India\",\"imageLink\":\"images/the-mahab-harata.jpg\",\"language\":\"Sanskrit\",\"link\":\"https://en.wikipedia.org/wiki/Mahabharata\\n\",\"pages\":276,\"title\":\"Mahabharata\",\"year\":-700},{\"author\":\"WaltWhitman\",\"country\":\"UnitedStates\",\"imageLink\":\"images/leaves-of-grass.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Leaves_of_Grass\\n\",\"pages\":152,\"title\":\"LeavesofGrass\",\"year\":1855},{\"author\":\"VirginiaWoolf\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/mrs-dalloway.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/Mrs_Dalloway\\n\",\"pages\":216,\"title\":\"MrsDalloway\",\"year\":1925},{\"author\":\"VirginiaWoolf\",\"country\":\"UnitedKingdom\",\"imageLink\":\"images/to-the-lighthouse.jpg\",\"language\":\"English\",\"link\":\"https://en.wikipedia.org/wiki/To_the_Lighthouse\\n\",\"pages\":209,\"title\":\"TotheLighthouse\",\"year\":1927},{\"author\":\"MargueriteYourcenar\",\"country\":\"France/Belgium\",\"imageLink\":\"images/memoirs-of-hadrian.jpg\",\"language\":\"French\",\"link\":\"https://en.wikipedia.org/wiki/Memoirs_of_Hadrian\\n\",\"pages\":408,\"title\":\"MemoirsofHadrian\",\"year\":1951}]";
